import { z } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import {
  CardContent,
  Form,
  FormControl,
  FormDescription,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
  Input,
} from '@/components/ui';
import { useEffect } from 'react';
import { useAuth } from '@/components/context';
import { SubmitButton } from '@/components/default';

const formSchema = z.object({
  full_name: z.string().min(5, {
    message: 'Full name is required & must be at least 5 characters.',
  }),
  email: z.string().email(),
});

type ProfileFormProps = {
  onSubmit: (values: z.infer<typeof formSchema>) => Promise<void>;
};

export const ProfileForm = ({ onSubmit }: ProfileFormProps) => {
  const { profile, isLoading } = useAuth();

  const form = useForm<z.infer<typeof formSchema>>({
    resolver: zodResolver(formSchema),
    defaultValues: {
      full_name: profile?.full_name ?? '',
      email: profile?.email ?? '',
    },
  });

  // Update form values when profile changes
  useEffect(() => {
    if (profile) {
      form.reset({
        full_name: profile.full_name ?? '',
        email: profile.email ?? '',
      });
    }
  }, [profile, form]);

  const handleSubmit = async (values: z.infer<typeof formSchema>) => {
    await onSubmit(values);
  };

  return (
    <CardContent>
      <Form {...form}>
        <form onSubmit={form.handleSubmit(handleSubmit)} className='space-y-6'>
          <FormField
            control={form.control}
            name='full_name'
            render={({ field }) => (
              <FormItem>
                <FormLabel>Full Name</FormLabel>
                <FormControl>
                  <Input {...field} />
                </FormControl>
                <FormDescription>Your public display name.</FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />

          <FormField
            control={form.control}
            name='email'
            render={({ field }) => (
              <FormItem>
                <FormLabel>Email</FormLabel>
                <FormControl>
                  <Input {...field} />
                </FormControl>
                <FormDescription>
                  Your email address associated with your account.
                </FormDescription>
                <FormMessage />
              </FormItem>
            )}
          />

          <div className='flex justify-center'>
            <SubmitButton disabled={isLoading} pendingText='Saving...'>
              Save Changes
            </SubmitButton>
          </div>
        </form>
      </Form>
    </CardContent>
  );
};
